const openpgp = require('openpgp');
const keyring = new openpgp.Keyring();

interface MutableKeyStore{
  loadKeyring: () => Promise<void>;
  importKeyPair: (publicKey: any, privateKey: any) => Promise<void>;
  importPublicKey: (publicKey: any) => Promise<void>;
  importPrivateKey: (privateKey: any) => Promise<void>;
  getPublicKeys: () => Array<any>;
  getTrustedKeys: () => Array<any>;
  getPrivateKeys: () => Array<any>;
  getPrivateKey: () => any;
  isValidKey: (key: any) => boolean;
  getFingerPrint: () => string;
  getKeyId: (key: any) => string;
  getKeysForId: (keyId: string) => Array<any>;
  getPublicKeyForId: (keyId: string) => any;
  getPrivateKeyForId: (keyId: string) => any;
  getPublicKeyForSubkeyId: (subkeyId: string) => any;
  getPublicKeysForAddress: (address: string) => Array<any>;
  removeKeysForId: (keyId: string) => Array<any>;
  removePublicKeyForId: (keyId: string) => any;
  removePublicKey: (publicKey: any) => any;
  clearKeysInKeyring: () => void;
}

class MutablePgpKeyStore implements MutableKeyStore{
  fingerprint: string;

  async loadKeyring(): Promise<void> {
    await keyring.load();
    // clear any keys already in the keychain
    // keyring.clear();
    await keyring.store();
  }

  async importKeyPair(publicKey: any, privateKey: any): Promise<void> {
    await keyring.publicKeys.importKey(publicKey);
    await keyring.privateKeys.importKey(privateKey);
  }

  async importPublicKey(publicKey: any): Promise<void> {
    await keyring.publicKeys.importKey(publicKey);
  }

  async importPrivateKey(privateKey: any): Promise<void> {
    await keyring.privateKeys.importKey(privateKey);
    this.fingerprint = keyring.privateKeys.keys[0].keyPacket.fingerprint;
  }

  getPublicKeys(): Array<any> {
    return keyring.publicKeys.keys;
  }

  getTrustedKeys(): Array<any> {
    return keyring.publicKeys.keys;
  }

  getPrivateKeys(): Array<any> {
    return keyring.privateKeys.keys;
  }

  getPrivateKey(): any {
    return keyring.privateKeys.keys[0];
  }

  isValidKey(key): boolean {
    return typeof key === openpgp.Key;
  }

  getFingerPrint(): string {
    return this.fingerprint;
  }

  getKeyId(key: any): string {
    return key.getKeyId().toHex();
  }

  getKeysForId(keyId: string): Array<any> {
    return keyring.getKeysForId(keyId);
  }

  getPublicKeyForId(keyId): any {
    return keyring.publicKeys.getForId(keyId);
  }

  getPrivateKeyForId(keyId): any {
    return keyring.privateKeys.getForId(keyId);
  }

  getPublicKeyForSubkeyId(subkeyId): any {
    return keyring.publicKeys.getForId(subkeyId, true);
  }

  getPublicKeysForAddress(address): Array<any> {
    return keyring.publicKeys.getForAddress(address);
  }

  removeKeysForId(keyId): Array<any> {
    return keyring.removeKeysForId(keyId);
  }

  removePublicKeyForId(keyId): any {
    return keyring.publicKeys.removeForId(keyId);
  }

  removePublicKey(publicKey: any): any {
    const keyId =  publicKey.getKeyId().toHex();
    return keyring.publicKeys.removeForId(keyId);
  }

  clearKeysInKeyring(): void {
    keyring.clear();
  }
}

export {
  MutablePgpKeyStore,
  MutableKeyStore
};
